@@ -8,6 +8,7 @@ package gosnmp
 
 import (
 	"testing"
+
 	"github.com/stretchr/testify/assert"
 )
 
@@ -112,3 +113,26 @@ func TestParseUint64(t *testing.T) {
 		}
 	}
 }
+
+func checkByteEquality2(a, b []byte) bool {
+
+	if a == nil && b == nil {
+		return true
+	}
+
+	if a == nil || b == nil {
+		return false
+	}
+
+	if len(a) != len(b) {
+		return false
+	}
+
+	for i := range a {
+		if a[i] != b[i] {
+			return false
+		}
+	}
+
+	return true
+}
