@@ -9,7 +9,9 @@ import (
 	"encoding/asn1"
 	"encoding/binary"
 	"fmt"
+	"io"
 	"net"
+	"strings"
 	"sync/atomic"
 	"time"
 )
@@ -36,7 +38,7 @@ type SnmpPacket struct {
 	Version            SnmpVersion
 	MsgFlags           SnmpV3MsgFlags
 	SecurityModel      SnmpV3SecurityModel
-	SecurityParameters SnmpV3SecurityParameters
+	SecurityParameters SnmpV3SecurityParameters // interface
 	ContextEngineID    string
 	ContextName        string
 	Community          string
@@ -49,7 +51,7 @@ type SnmpPacket struct {
 	NonRepeaters       uint8
 	MaxRepetitions     uint8
 	Variables          []SnmpPDU
-	Logger             Logger
+	Logger             Logger // interface
 
 	// v1 traps have a very different format from v2c and v3 traps.
 	//
@@ -122,26 +124,31 @@ func (x *GoSNMP) logPrintf(format string, v ...interface{}) {
 // send/receive one snmp request
 func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 	wait bool) (result *SnmpPacket, err error) {
-	finalDeadline := time.Now().Add(x.Timeout)
-
 	allReqIDs := make([]uint32, 0, x.Retries+1)
-	allMsgIDs := make([]uint32, 0, x.Retries+1)
+	// allMsgIDs := make([]uint32, 0, x.Retries+1) // unused
+
+	timeout := x.Timeout
 	for retries := 0; ; retries++ {
 		if retries > 0 {
 			x.logPrintf("Retry number %d. Last error was: %v", retries, err)
-			if time.Now().After(finalDeadline) {
-				err = fmt.Errorf("Request timeout (after %d retries)", retries-1)
-				break
+			if x.ExponentialTimeout {
+				// https://www.webnms.com/snmp/help/snmpapi/snmpv3/v1/timeout.html
+				timeout *= 2
 			}
 			if retries > x.Retries {
-				// Report last error
+				if strings.Contains(err.Error(), "timeout") {
+					err = fmt.Errorf("Request timeout (after %d retries)", retries-1)
+				}
 				break
 			}
 		}
 		err = nil
 
-		reqDeadline := time.Now().Add(x.Timeout / time.Duration(x.Retries+1))
-		x.Conn.SetDeadline(reqDeadline)
+		reqDeadline := time.Now().Add(timeout)
+		err = x.Conn.SetDeadline(reqDeadline)
+		if err != nil {
+			return nil, err
+		}
 
 		// Request ID is an atomic counter (started at a random value)
 		reqID := atomic.AddUint32(&(x.requestID), 1) // TODO: fix overflows
@@ -151,7 +158,7 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 
 		if x.Version == Version3 {
 			msgID := atomic.AddUint32(&(x.msgID), 1) // TODO: fix overflows
-			allMsgIDs = append(allMsgIDs, msgID)
+			// allMsgIDs = append(allMsgIDs, msgID) // unused
 
 			packetOut.MsgID = msgID
 
@@ -180,7 +187,7 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 		}
 
 		// all sends wait for the return packet, except for SNMPv2Trap
-		if wait == false {
+		if !wait {
 			return &SnmpPacket{}, nil
 		}
 
@@ -192,11 +199,21 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 
 			var resp []byte
 			resp, err = x.receive()
-			if err != nil {
+			if err == io.EOF && strings.HasPrefix(x.Transport, "tcp") {
+				// EOF on TCP: reconnect and retry. Do not count
+				// as retry as socket was broken
+				x.logPrintf("ERROR: EOF. Performing reconnect")
+				err = x.netConnect()
+				if err != nil {
+					return nil, err
+				}
+				retries--
+				break
+			} else if err != nil {
 				// receive error. retrying won't help. abort
 				break
 			}
-			x.logPrint("GET RESPONSE OK : %+v", resp)
+			x.logPrintf("GET RESPONSE OK: %+v", resp)
 			result = new(SnmpPacket)
 			result.Logger = x.Logger
 
@@ -256,7 +273,7 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 				validID = true
 			}
 			if !validID {
-				x.logPrint("ERROR  out of order ")
+				x.logPrint("ERROR  out of order")
 				err = fmt.Errorf("Out of order response")
 				continue
 			}
@@ -314,7 +331,7 @@ func (x *GoSNMP) send(packetOut *SnmpPacket, wait bool) (result *SnmpPacket, err
 
 		// detect out-of-time-window error and retransmit with updated auth engine parameters
 		if len(result.Variables) == 1 && result.Variables[0].Name == ".1.3.6.1.6.3.15.1.1.2.0" {
-			x.logPrintf("WARNING detected out-of-time-window ERROR")
+			x.logPrint("WARNING detected out-of-time-window ERROR")
 			err = x.updatePktSecurityParameters(packetOut)
 			if err != nil {
 				x.logPrintf("ERROR  updatePktSecurityParameters error: %s", err)
@@ -326,7 +343,7 @@ func (x *GoSNMP) send(packetOut *SnmpPacket, wait bool) (result *SnmpPacket, err
 
 	// detect unknown engine id error and retransmit with updated engine id
 	if len(result.Variables) == 1 && result.Variables[0].Name == ".1.3.6.1.6.3.15.1.1.4.0" {
-		x.logPrintf("WARNING detected unknown enginer id ERROR")
+		x.logPrint("WARNING detected unknown enginer id ERROR")
 		err = x.updatePktSecurityParameters(packetOut)
 		if err != nil {
 			x.logPrintf("ERROR  updatePktSecurityParameters error: %s", err)
@@ -338,6 +355,8 @@ func (x *GoSNMP) send(packetOut *SnmpPacket, wait bool) (result *SnmpPacket, err
 }
 
 // -- Marshalling Logic --------------------------------------------------------
+
+// MarshalMsg marshalls a snmp packet, ready for sending across the wire
 func (packet *SnmpPacket) MarshalMsg() ([]byte, error) {
 	return packet.marshalMsg()
 }
@@ -376,7 +395,10 @@ func (packet *SnmpPacket) marshalMsg() ([]byte, error) {
 		return nil, err2
 	}
 	msg.Write(bufLengthBytes)
-	buf.WriteTo(msg) // reverse logic - want to do msg.Write(buf)
+	_, err = buf.WriteTo(msg)
+	if err != nil {
+		return nil, err
+	}
 
 	authenticatedMessage, err := packet.authenticate(msg.Bytes())
 	if err != nil {
@@ -389,33 +411,46 @@ func (packet *SnmpPacket) marshalMsg() ([]byte, error) {
 func (packet *SnmpPacket) marshalSNMPV1TrapHeader() ([]byte, error) {
 	buf := new(bytes.Buffer)
 
-	mOid, err := marshalOID(packet.Enterprise)
+	// marshal OID
+	oidBytes, err := marshalOID(packet.Enterprise)
 	if err != nil {
-		return nil, fmt.Errorf("Unable to marshal OID: %s\n", err.Error())
+		return nil, fmt.Errorf("unable to marshal OID: %s", err.Error())
 	}
+	buf.Write([]byte{byte(ObjectIdentifier), byte(len(oidBytes))})
+	buf.Write(oidBytes)
 
-	buf.Write([]byte{ObjectIdentifier, byte(len(mOid))})
-	buf.Write(mOid)
-
-	// write IPAddress type, length and ipAddress value
+	// marshal AgentAddress (ip address)
 	ip := net.ParseIP(packet.AgentAddress)
 	ipAddressBytes := ipv4toBytes(ip)
-	buf.Write([]byte{IPAddress, byte(len(ipAddressBytes))})
+	buf.Write([]byte{byte(IPAddress), byte(len(ipAddressBytes))})
 	buf.Write(ipAddressBytes)
 
-	buf.Write([]byte{Integer, 1})
-	buf.WriteByte(byte(packet.GenericTrap))
-
-	buf.Write([]byte{Integer, 1})
-	buf.WriteByte(byte(packet.SpecificTrap))
-
-	timeTicks, e := marshalUint32(uint32(packet.Timestamp))
-	if e != nil {
-		return nil, fmt.Errorf("Unable to Timestamp: %s\n", e.Error())
+	// marshal GenericTrap. Could just cast GenericTrap to a single byte as IDs greater than 6 are unknown,
+	// but do it properly. See issue 182.
+	var genericTrapBytes []byte
+	genericTrapBytes, err = marshalInt32(packet.GenericTrap)
+	if err != nil {
+		return nil, fmt.Errorf("Unable to marshal SNMPv1 GenericTrap: %s", err.Error())
 	}
+	buf.Write([]byte{byte(Integer), byte(len(genericTrapBytes))})
+	buf.Write(genericTrapBytes)
 
-	buf.Write([]byte{TimeTicks, byte(len(timeTicks))})
-	buf.Write(timeTicks)
+	// marshal SpecificTrap
+	var specificTrapBytes []byte
+	specificTrapBytes, err = marshalInt32(packet.SpecificTrap)
+	if err != nil {
+		return nil, fmt.Errorf("Unable to marshal SNMPv1 SpecificTrap: %s", err.Error())
+	}
+	buf.Write([]byte{byte(Integer), byte(len(specificTrapBytes))})
+	buf.Write(specificTrapBytes)
+
+	// marshal timeTicks
+	timeTickBytes, e := marshalUint32(uint32(packet.Timestamp))
+	if e != nil {
+		return nil, fmt.Errorf("unable to Timestamp: %s", e.Error())
+	}
+	buf.Write([]byte{byte(TimeTicks), byte(len(timeTickBytes))})
+	buf.Write(timeTickBytes)
 
 	return buf.Bytes(), nil
 }
@@ -454,7 +489,7 @@ func (packet *SnmpPacket) marshalPDU() ([]byte, error) {
 		err := binary.Write(buf, binary.BigEndian, packet.RequestID)
 
 		if err != nil {
-			return nil, fmt.Errorf("Unable to marshal OID: %s\n", err.Error())
+			return nil, fmt.Errorf("unable to marshal OID: %s", err.Error())
 		}
 
 		// error
@@ -482,7 +517,10 @@ func (packet *SnmpPacket) marshalPDU() ([]byte, error) {
 	}
 	pdu.Write(bufLengthBytes)
 
-	buf.WriteTo(pdu) // reverse logic - want to do pdu.Write(buf)
+	_, err = buf.WriteTo(pdu)
+	if err != nil {
+		return nil, err
+	}
 	return pdu.Bytes(), nil
 }
 
@@ -531,7 +569,7 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		tmpBuf.Write([]byte{byte(ObjectIdentifier)})
 		tmpBuf.Write(ltmp)
 		tmpBuf.Write(oid)
-		tmpBuf.Write([]byte{Null, 0x00})
+		tmpBuf.Write([]byte{byte(Null), byte(EndOfContents)})
 
 		ltmp, err = marshalLength(tmpBuf.Len())
 		if err != nil {
@@ -539,7 +577,10 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		}
 		pduBuf.Write([]byte{byte(Sequence)})
 		pduBuf.Write(ltmp)
-		tmpBuf.WriteTo(pduBuf)
+		_, err = tmpBuf.WriteTo(pduBuf)
+		if err != nil {
+			return nil, err
+		}
 
 	case Integer:
 		// Oid
@@ -555,7 +596,7 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 			intBytes, err = marshalInt32(value)
 			pdu.Check(err)
 		default:
-			return nil, fmt.Errorf("Unable to marshal PDU Integer; not byte or int.")
+			return nil, fmt.Errorf("unable to marshal PDU Integer; not byte or int")
 		}
 		tmpBuf.Write([]byte{byte(Integer), byte(len(intBytes))})
 		tmpBuf.Write(intBytes)
@@ -600,7 +641,7 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		case string:
 			octetStringBytes = []byte(value)
 		default:
-			return nil, fmt.Errorf("Unable to marshal PDU OctetString; not []byte or String.")
+			return nil, fmt.Errorf("unable to marshal PDU OctetString; not []byte or String")
 		}
 
 		var length []byte
@@ -665,7 +706,7 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 			ip := net.ParseIP(value)
 			ipAddressBytes = ipv4toBytes(ip)
 		default:
-			return nil, fmt.Errorf("Unable to marshal PDU IPAddress; not []byte or String.")
+			return nil, fmt.Errorf("unable to marshal PDU IPAddress; not []byte or String")
 		}
 		tmpBuf.Write([]byte{byte(IPAddress), byte(len(ipAddressBytes))})
 		tmpBuf.Write(ipAddressBytes)
@@ -698,12 +739,12 @@ func (x *GoSNMP) unmarshalHeader(packet []byte, response *SnmpPacket) (int, erro
 
 	// First bytes should be 0x30
 	if PDUType(packet[0]) != Sequence {
-		return 0, fmt.Errorf("Invalid packet header\n")
+		return 0, fmt.Errorf("invalid packet header")
 	}
 
 	length, cursor := parseLength(packet)
 	if len(packet) != length {
-		return 0, fmt.Errorf("Error verifying packet sanity: Got %d Expected: %d\n", len(packet), length)
+		return 0, fmt.Errorf("error verifying packet sanity: Got %d Expected: %d", len(packet), length)
 	}
 	x.logPrintf("Packet sanity verified, we got all the bytes (%d)", length)
 
@@ -768,7 +809,7 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 
 	getResponseLength, cursor := parseLength(packet)
 	if len(packet) != getResponseLength {
-		return fmt.Errorf("Error verifying Response sanity: Got %d Expected: %d\n", len(packet), getResponseLength)
+		return fmt.Errorf("error verifying Response sanity: Got %d Expected: %d", len(packet), getResponseLength)
 	}
 	x.logPrintf("getResponseLength: %d", getResponseLength)
 
@@ -835,7 +876,7 @@ func (x *GoSNMP) unmarshalTrapV1(packet []byte, response *SnmpPacket) error {
 
 	getResponseLength, cursor := parseLength(packet)
 	if len(packet) != getResponseLength {
-		return fmt.Errorf("Error verifying Response sanity: Got %d Expected: %d\n", len(packet), getResponseLength)
+		return fmt.Errorf("error verifying Response sanity: Got %d Expected: %d", len(packet), getResponseLength)
 	}
 	x.logPrintf("getResponseLength: %d", getResponseLength)
 
@@ -908,7 +949,7 @@ func (x *GoSNMP) unmarshalVBL(packet []byte, response *SnmpPacket) error {
 
 	vblLength, cursor = parseLength(packet)
 	if len(packet) != vblLength {
-		return fmt.Errorf("Error verifying: packet length %d vbl length %d\n", len(packet), vblLength)
+		return fmt.Errorf("error verifying: packet length %d vbl length %d", len(packet), vblLength)
 	}
 	x.logPrintf("vblLength: %d", vblLength)
 
@@ -956,8 +997,10 @@ func (x *GoSNMP) unmarshalVBL(packet []byte, response *SnmpPacket) error {
 // receive response from network and read into a byte array
 func (x *GoSNMP) receive() ([]byte, error) {
 	n, err := x.Conn.Read(x.rxBuf[:])
-	if err != nil {
-		return nil, fmt.Errorf("Error reading from UDP: %s", err.Error())
+	if err == io.EOF {
+		return nil, err
+	} else if err != nil {
+		return nil, fmt.Errorf("Error reading from socket: %s", err.Error())
 	}
 
 	if n == rxBufSize {
