@@ -56,6 +56,9 @@ func (x *GoSNMP) validateParametersV3() error {
 	if x.SecurityModel != UserSecurityModel {
 		return fmt.Errorf("The SNMPV3 User Security Model is the only SNMPV3 security model currently implemented")
 	}
+	if x.SecurityParameters == nil {
+		return fmt.Errorf("SNMPV3 SecurityParameters must be set")
+	}
 
 	return x.SecurityParameters.validate(x.MsgFlags)
 }
@@ -297,7 +300,7 @@ func (x *GoSNMP) unmarshalV3Header(packet []byte,
 	response *SnmpPacket) (int, error) {
 
 	if PDUType(packet[cursor]) != Sequence {
-		return 0, fmt.Errorf("Invalid SNMPV3 Header\n")
+		return 0, fmt.Errorf("invalid SNMPV3 Header")
 	}
 
 	_, cursorTmp := parseLength(packet[cursor:])
@@ -343,8 +346,8 @@ func (x *GoSNMP) unmarshalV3Header(packet []byte,
 		x.logPrintf("Parsed security model %d", SecModel)
 	}
 
-	if PDUType(packet[cursor]) != OctetString {
-		return 0, fmt.Errorf("Invalid SNMPV3 Security Parameters\n")
+	if PDUType(packet[cursor]) != PDUType(OctetString) {
+		return 0, fmt.Errorf("invalid SNMPV3 Security Parameters")
 	}
 	_, cursorTmp = parseLength(packet[cursor:])
 	cursor += cursorTmp
@@ -364,7 +367,7 @@ func (x *GoSNMP) decryptPacket(packet []byte, cursor int, response *SnmpPacket)
 	var decrypted = false
 
 	switch PDUType(packet[cursor]) {
-	case OctetString:
+	case PDUType(OctetString):
 		// pdu is encrypted
 		packet, err = response.SecurityParameters.decryptPacket(packet, cursor)
 		if err != nil {
@@ -401,7 +404,7 @@ func (x *GoSNMP) decryptPacket(packet []byte, cursor int, response *SnmpPacket)
 		}
 
 	default:
-		return nil, 0, fmt.Errorf("Error parsing SNMPV3 scoped PDU\n")
+		return nil, 0, fmt.Errorf("error parsing SNMPV3 scoped PDU")
 	}
 	return packet, cursor, nil
 }
